/*
 * DepthFirstSearchAction.cpp
 *
 *  Created on: May 9, 2010
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "Helpers/MemDebug.hpp"

#include "Actions/FragmentationAction/DepthFirstSearchAction.hpp"
#include "Actions/ActionRegistry.hpp"
#include "atom.hpp"
#include "bondgraph.hpp"
#include "config.hpp"
#include "Helpers/Log.hpp"
#include "molecule.hpp"
#include "Descriptors/MoleculeDescriptor.hpp"
#include "Descriptors/MoleculeIdDescriptor.hpp"
#include "stackclass.hpp"
#include "Helpers/Verbose.hpp"
#include "World.hpp"

#include <iostream>
#include <string>

using namespace std;

#include "UIElements/UIFactory.hpp"
#include "UIElements/Dialog.hpp"
#include "Actions/ValueStorage.hpp"

const char FragmentationDepthFirstSearchAction::NAME[] = "depth-first-search";

FragmentationDepthFirstSearchAction::FragmentationDepthFirstSearchAction() :
  Action(NAME)
{}

FragmentationDepthFirstSearchAction::~FragmentationDepthFirstSearchAction()
{}

void FragmentationDepthFirstSearch(double distance) {
  ValueStorage::getInstance().setCurrentValue(FragmentationDepthFirstSearchAction::NAME, distance);
  ActionRegistry::getInstance().getActionByName(FragmentationDepthFirstSearchAction::NAME)->call(Action::NonInteractive);
};

Dialog* FragmentationDepthFirstSearchAction::fillDialog(Dialog *dialog) {
  ASSERT(dialog,"No Dialog given when filling action dialog");

  dialog->queryDouble(NAME, ValueStorage::getInstance().getDescription(NAME));

  return dialog;
}

Action::state_ptr FragmentationDepthFirstSearchAction::performCall() {
  double distance;

  ValueStorage::getInstance().queryCurrentValue(NAME, distance);

  DoLog(1) && (Log() << Verbose(1) << "Depth-First-Search Analysis." << endl);
  molecule * const mol = World::getInstance().getMolecule(MoleculeById(0));
  MoleculeLeafClass *Subgraphs = NULL;      // list of subgraphs from DFS analysis
  int *MinimumRingSize = new int[mol->getAtomCount()];
  atom **ListOfAtoms = NULL;
  class StackClass<bond *> *BackEdgeStack = NULL;
  class StackClass<bond *> *LocalBackEdgeStack = NULL;
  mol->CreateAdjacencyList(distance, World::getInstance().getConfig()->GetIsAngstroem(), &BondGraph::CovalentMinMaxDistance, NULL);
  Subgraphs = mol->DepthFirstSearchAnalysis(BackEdgeStack);
  if (Subgraphs != NULL) {
    int FragmentCounter = 0;
    while (Subgraphs->next != NULL) {
      Subgraphs = Subgraphs->next;
      ListOfAtoms = NULL;
      Subgraphs->FillBondStructureFromReference(mol, ListOfAtoms, false);  // we want to keep the created ListOfLocalAtoms
      LocalBackEdgeStack = new StackClass<bond *> (Subgraphs->Leaf->BondCount);
      Subgraphs->Leaf->PickLocalBackEdges(ListOfAtoms, BackEdgeStack, LocalBackEdgeStack);
      Subgraphs->Leaf->CyclicStructureAnalysis(LocalBackEdgeStack, MinimumRingSize);
      delete(LocalBackEdgeStack);
      delete(Subgraphs->previous);
      delete[](ListOfAtoms);  // and here we remove it
      FragmentCounter++;
    }
    delete(Subgraphs);
  }
  delete(BackEdgeStack);
  delete[](MinimumRingSize);
  return Action::success;
}

Action::state_ptr FragmentationDepthFirstSearchAction::performUndo(Action::state_ptr _state) {
  return Action::success;
}

Action::state_ptr FragmentationDepthFirstSearchAction::performRedo(Action::state_ptr _state){
  return Action::success;
}

bool FragmentationDepthFirstSearchAction::canUndo() {
  return true;
}

bool FragmentationDepthFirstSearchAction::shouldUndo() {
  return true;
}

const string FragmentationDepthFirstSearchAction::getName() {
  return NAME;
}
